<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if( defined('WP_ROCKET_VERSION') && !class_exists('ReyCore_Compatibility__WPRocket') ):

	class ReyCore_Compatibility__WPRocket
	{
		public function __construct()
		{
			add_filter( 'rocket_cdn_reject_files', [$this, 'exclude_files_cdn'], 20, 2);
			add_filter( 'rocket_excluded_inline_js_content', [$this, 'exclude_inline_js'], 10);
			// add_filter('rocket_exclude_defer_js', [$this, 'exclude_defer_js'], 10);
			// add_filter('rocket_defer_inline_exclusions', [$this, 'exclude_inline_defer_js'], 10);
			add_filter( 'rocket_cpcss_excluded_post_types', [$this, 'exclude_cpt_cpcss'], 10);
			add_action( 'rocket_critical_css_generation_process_complete', [$this, 'add_extra_cpcss']);
			add_action( 'reycore/kirki_fields/after_field=perf__preload_assets', [ $this, 'add_customizer_options' ] );
			add_action( 'rey/flush_cache_after_updates', [ $this, 'empty_theme_mod' ] );
			add_filter( 'reycore/elementor/revslider/waitforinit', [ $this, 'revslider_defer' ] );
			add_filter( 'reycore/is_mobile', [ $this, 'is_mobile' ] );
			add_filter( 'reycore/supports_mobile_caching', [ $this, 'cache_separately' ] );
			add_filter( 'reycore/mobile_improvements', [ $this, 'supports_mobile_improvements' ] );
			add_filter( 'theme_mod_site_preloader', [ $this, 'disable_site_preloader' ] );
			add_action( 'reycore/kirki_fields/before_field=site_preloader', [ $this, 'add_preloader_option_notice' ] );
			add_action( 'wp_footer', [ $this, 'enqueue_delay_js_handler' ] );
			add_filter( 'rocket_delay_js_exclusions', [$this, 'exclude_delay_js'], 10);
			add_filter( 'reycore/delay_js', [$this, 'rey_delay_js']);
			add_filter( 'reycore/delay_script_params', [ $this, 'hint_delay_event' ] );
			add_filter( 'theme_mod_loop_animate_in', [ $this, 'disable_loop_animation_delay_js' ] );
			add_filter( 'rocket_usedcss_content', [ $this, 'get_fonts_css' ] );
			add_filter( 'reycore/critical_css/disable', [ $this, 'disable_rey_critical' ] );
			add_filter( 'woocommerce_single_product_image_gallery_classes', [$this, 'disable_pdp_gallery_loading'], 100);

		}

		public function exclude_files_cdn($files) {

			// (.*).svg

			if( function_exists('rey__svg_sprite_path') ){
				$files[] = rey__svg_sprite_path();
			}

			if( function_exists('reycore__icons_sprite_path') ){
				$files[] = reycore__icons_sprite_path();
			}

			if( function_exists('reycore__social_icons_sprite_path') ){
				$files[] = reycore__social_icons_sprite_path();
			}

			return $files;
		}

		function exclude_defer_js( $pattern ) {

			return $pattern;
		}

		function exclude_inline_defer_js( $pattern ) {

			if( is_array($pattern) ){
				// $pattern[] = 'revapi';
			}
			else {
				// $pattern .= '|revapi';
			}

			return $pattern;
		}

		function exclude_inline_js( $pattern ) {

			$pattern[] = 'rey-no-js';

			return $pattern;
		}

		function exclude_cpt_cpcss( $cpt ) {

			if( class_exists('ReyCore_GlobalSections') ){
				$cpt[] = ReyCore_GlobalSections::POST_TYPE;
			}

			return $cpt;
		}

		function revslider_defer( $status ) {

			// defer is disabled, can return predefined
			if ( ! get_rocket_option( 'defer_all_js' ) ) {
				return $status;
			}

			// defer per post is disabled, can return predefined
			if ( is_rocket_post_excluded_option( 'defer_all_js' ) ) {
				return $status;
			}

			return false;
		}

		function disable_rey_critical(){
			return get_rocket_option( 'async_css' );
		}

		function add_extra_cpcss(){

			$css = function_exists('reyCoreCriticalCss') ? reyCoreCriticalCss()->css() : [];

			$css[] = get_theme_mod('perf__wprocket_extra_critical', '');

			if( !($filesystem = reycore__wp_filesystem()) ){
				return;
			}

			$dir_path = WP_CONTENT_DIR . '/cache/critical-css/' . (is_multisite() ? get_current_blog_id() : 1);

			if ( ! $filesystem->is_dir( $dir_path ) ) {
				return;
			}

			if( ! ($list = $filesystem->dirlist( $dir_path )) ){
				return;
			}

			$css_files = array_filter($list, function($file){
				return ($info = pathinfo($file['name'])) && $info['extension'] === 'css';
			});

			if( empty($css_files) ){
				return;
			}

			$success = [];

			foreach ($css_files as $key => $css_file) {

				$file_path = trailingslashit($dir_path) . $key;

				$data = $filesystem->get_contents( $file_path );
				$data .= implode('', $css);

				if( $filesystem->put_contents( $file_path, $data ) ){
					$success[] = true;
				}

			}

			if( in_array(true, $success, true) ){
				rocket_clean_domain();
			}
		}

		function empty_theme_mod(){
			set_theme_mod('perf__wprocket_extra_critical', '');
		}

		function is_mobile( $status ){

			if ( ! class_exists( 'WP_Rocket_Mobile_Detect' ) ) {
				return $status;
			}

			$detect = new WP_Rocket_Mobile_Detect();

			if ( $detect->isMobile() ) {
				return true;
			}

			return $status;
		}

		function cache_separately( $status ){

			if( get_rocket_option( 'cache_mobile' ) && get_rocket_option( 'do_caching_mobile_files' ) ){
				return true;
			}

			return $status;
		}

		function supports_mobile_improvements( $status ){

			if( get_theme_mod('perf__wprocket_mobile_improvements', false) ){
				return true;
			}

			return $status;
		}

		function add_customizer_options( $args ){

			ReyCoreKirki::add_field( 'rey_core_kirki', [
				'type'        => 'textarea',
				'settings'    => 'perf__wprocket_extra_critical',
				'label'       => esc_html_x( 'Extra Critical CSS styles', 'Customizer control title', 'rey-core' ),
				'description' => esc_html_x( 'Append extra styles to WPRocket\'s CPCSS in case something is not rendering properly. Works only if "Optimize CSS delivery" is enabled.', 'Customizer control description', 'rey-core' ),
				'section'     => $args['section'],
				'default'     => '',
				'input_attrs'     => [
					'placeholder' => esc_html_x('eg: .selector {}', 'Customizer control description', 'rey-core'),
					'data-control-class' => '--block-label',
				],
			] );

			ReyCoreKirki::add_field( 'rey_core_kirki', [
				'type'        => 'toggle',
				'settings'    => 'perf__wprocket_mobile_improvements',
				'label'       => esc_html__( 'Add Mobile Improvements', 'rey-core' ),
				'description' => sprintf(_x( 'Requires WPRocket Mobile Cache & Cache Separately to be enabled. Please <a href="%s" target="_blank">read more here</a> on how to use this option.', 'Customizer control description', 'rey-core' ), 'https://support.reytheme.com/kb/mobile-improvements/'),
				'section'     => $args['section'],
				'default'     => false,
			] );

		}

		function is_javascript_delayed(){

			if( defined('WP_ROCKET_VERSION') ){
				if ( version_compare( WP_ROCKET_VERSION, '3.9', '<' ) ) {
					return;
				}
			}

			if( defined('DONOTCACHEPAGE') && DONOTCACHEPAGE ){
				return;
			}

			if ( function_exists('rocket_bypass') && rocket_bypass() ) {
				return false;
			}

			if( defined('DONOTROCKETOPTIMIZE') && DONOTROCKETOPTIMIZE ){
				return;
			}

			if ( function_exists('is_rocket_post_excluded_option') && is_rocket_post_excluded_option( 'delay_js' ) ) {
				return false;
			}

			return get_rocket_option( 'delay_js' );
		}

		function rey_delay_js($status){

			if ( $this->is_javascript_delayed() ) {
				return true;
			}

			return $status;
		}

		function hint_delay_event($params){

			if ( $this->is_javascript_delayed() ) {
				$params['event'] = 'rocket-allScriptsLoaded';
			}

			return $params;
		}

		function disable_loop_animation_delay_js($mod){

			if ( $this->is_javascript_delayed() ) {
				return false;
			}

			return $mod;
		}

		function disable_pdp_gallery_loading($classes){
			if ( $this->is_javascript_delayed() ) {
				unset($classes['loading']);
			}
			return $classes;
		}

		function exclude_delay_js( $pattern ) {

			/**
			 * Excluding scripts beats the purpose of this option.
			 * When an exclusion is added, its dependencies must also be added and it's jQuery, Rey Core, Elementor frontend, etc.
			 */

			$pattern[] = 'reycore-delay-js';
			$pattern[] = 'rey-set-xl';
			$pattern[] = 'reycore-critical-css-js';
			$pattern[] = 'rey-no-js';

			return $pattern;
		}

		function enqueue_delay_js_handler(){
			if ( $this->is_javascript_delayed() ) {
				wp_enqueue_script('reycore-delay-js');
			}
		}

		function disable_site_preloader($mod){

			if ( $this->is_javascript_delayed() ) {
				return false;
			}

			return $mod;
		}

		function add_preloader_option_notice($args){

			if ( $this->is_javascript_delayed() ) {
				reycore_customizer__notice([
					'section'     => $args['section'],
					'default'     => esc_html_x('Heads up! The preloader is disabled because WPRocket\'s Delay JS option is enabled.', ' Customizer control label', 'rey-core')
				] );
			}

		}

		function unused_css(){
			return '
.screen-reader-text
.--active-tab
.--tabs-loaded
.--animated-in
.header-overlay--is-opened
.site-overlay--is-opened
.--overlay-darken
.search-inline--active
.search-panel--is-opened
.header-account--active
.--cart-active
.--side-panel-active
.--hover
[aria-expanded="true"]
.--submenu-indicator(.*)
.rey-searchAjax
.rey-searchResults
.rey-searchItem(.*)
.--loading
.woocommerce-js
.rey-crossSells(.*)
.rey-cartPanel
.--in-wishlist
.rey-wishlistItem(.*)
.rey-quickviewPanel
.qty
.--filter-panel-active
.rey-wishlist-notice-wrapper
.rey-compare-notice-wrapper
.--visible
.btn-primary
.btn-secondary
.alt
.--no-scroll
.--mobileNav--active
.rey-acPopup(.*)
.rey-cartRecent(.*)
.noUi(.*)
ol.commentlist
.star-rating
.meta
.avatar
.comment-text
.rey-productLoop-variationsForm
div.product
.rey-productSummary
.--offcanvas(.*)
.--is-active
.--active
.is-active
.--cSslide-active
.--cNest-active
.--initialised
.--init
.--animate-out
.--is-filtering
.--empty
.--cover-split-header(.*)
			';

			return [];
		}

		public function get_fonts_css( $css ) {

			if( ! function_exists('reyCoreWebfonts') ){
				return $css;
			}

			return implode('', reyCoreWebfonts()->enqueue_css()) . $css;
		}

	}

	new ReyCore_Compatibility__WPRocket;
endif;
