<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

if( !class_exists('ReyCore_Element_Kit') ):
    /**
	 * Kit Overrides and customizations
	 *
	 * @since 1.0.0
	 */
	class ReyCore_Element_Kit {

		function __construct(){
			add_action( 'elementor/element/kit/section_settings-layout/before_section_end', [$this, 'kit_layout_settings'], 10);
			add_action( 'elementor/element/kit/section_layout-settings/before_section_end', [$this, 'kit_layout_settings'], 10);
			add_action( 'elementor/element/kit/section_buttons/before_section_end', [$this, 'kit_button_settings'], 10);
			// add_action( 'elementor/element/kit/section_typography/before_section_end', [$this, 'kit_typography_settings'], 10);
			// add_action('elementor/db/before_save', [$this , 'force_theme_mods'], 10, 2);
		}

		/**
		 * Remove Container width as it directly conflicts with Rey's container settings
		 *
		 * @since 1.6.12
		 */
		function kit_layout_settings( $element ){
			$element->remove_control( 'container_width' );
			$element->remove_control( 'container_width_tablet' );
			$element->remove_control( 'container_width_mobile' );
		}

		function kit_button_settings( $element ){

			if( ! apply_filters('reycore/elementor/kit/custom_button_selectors', true) ){
				return;
			}

			$button_selectors = [
				'{{WRAPPER}} .elementor-button',
			];

			$button_hover_selectors = [
				'{{WRAPPER}} .elementor-button:hover',
				'{{WRAPPER}} .elementor-button:focus',
			];

			$button_selector = implode( ',', $button_selectors );
			$button_hover_selector = implode( ',', $button_hover_selectors );

			$controls = [
				// default
				'button_text_color' => $button_selector,
				'button_background_color' => $button_selector,
				'button_box_shadow_box_shadow' => $button_selector,
				'button_border_radius' => $button_selector,
				'button_border_width' => $button_selector,
				'button_border_color' => $button_selector,
				'button_padding' => $button_selector,
				// hover
				'button_hover_text_color' => $button_hover_selector,
				'button_hover_background_color' => $button_hover_selector,
				'button_hover_border_width' => $button_hover_selector,
				'button_hover_border_color' => $button_hover_selector,
				'button_hover_box_shadow_box_shadow' => $button_hover_selector,
				'button_hover_border_radius' => $button_hover_selector,
			];

			foreach ($controls as $control_key => $control_selector) {

				$kit_element = \Elementor\Plugin::instance()->controls_manager->get_control_from_stack( $element->get_unique_name(), $control_key );

				$old_selector = array_values($kit_element['selectors']);

				if( ! isset($old_selector[0]) ){
					continue;
				}

				$kit_element['selectors'] = [];
				$kit_element['selectors'][$control_selector] = $old_selector[0];

				$element->update_control( $control_key, $kit_element );
			}

		}

		/**
		 * Add options into Typography
		 *
		 * @since 1.9.6
		 */
		function kit_typography_settings( $element ){

			$element->start_injection( [
				'of' => 'paragraph_spacing_mobile',
			] );

			/**
			 * Add option to disable Rey's Customizer typography controls
			 */
			$element->add_control(
				'disable_rey_typography',
				[
					'label' => esc_html__( 'Disable Typography in Customizer', 'rey-core' ) . reyCoreElementor::getReyBadge(),
					'type' => \Elementor\Controls_Manager::SWITCHER,
					'default' => get_theme_mod('typography_inherit_elementor', false),
				]
			);

			$element->end_injection();
		}

		/**
		 * Disable Rey's Customizer typography controls
		 *
		 * @since 1.9.6
		 */
		public function force_theme_mods( $post_id , $is_meta ) {

			global $post;

			if( ! ($elementor_meta = get_post_meta( $post->ID, \Elementor\Core\Base\Document::PAGE_META_KEY, true )) ){
				return;
			}

			if( isset($elementor_meta['disable_rey_typography']) && $elementor_meta['disable_rey_typography'] === 'yes' ){
				set_theme_mod('typography_inherit_elementor', true);
			}
        }

	}
endif;
