<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

if(!class_exists('ReyCore_Widget_Carousel') && class_exists('\\ReyCore\\Cards\\CardElement')):


	class ReyCore_Widget_Carousel extends \ReyCore\Cards\CardElement {

		public $_settings = [];

		public $_items = [];

		public function get_name() {
			return 'reycore-carousel';
		}

		public function get_title() {
			return __( 'Carousel', 'rey-core' );
		}

		public function get_icon() {
			return 'eicon-posts-carousel';
		}

		public function get_categories() {
			return [ 'rey-theme' ];
		}

		public function rey_get_script_depends() {
			return [ 'rey-splide', 'imagesloaded', 'reycore-widget-carousel-scripts' ];
		}

		public function get_keywords() {
			return [ 'carousel', 'slider', 'posts', 'gallery', 'categories' ];
		}

		public function get_custom_help_url() {
			return 'https://support.reytheme.com/kb/rey-elements/#carousel';
		}

		public function add_element_controls() {

			$this->selectors['carousel'] = '{{WRAPPER}} .rey-carouselEl';
			$this->selectors['arrows'] = '{{WRAPPER}} .__arrows';
			$this->selectors['arrow'] = '{{WRAPPER}} .rey-arrowSvg';

			$this->controls__content();
			$this->controls__images();
			$this->controls__custom_content();
			$this->controls__post_query();
			$this->controls__product_cat_query();
			$this->controls__carousel_settings();
			$this->controls__content_styles();
			$this->controls__media_styles();
			$this->controls__title_styles();
			$this->controls__subtitle_styles();
			$this->controls__label_styles();
			$this->controls__button_styles();
			$this->controls__arrow_styles();
			$this->controls__dots_styles();
		}

		public function controls__carousel_settings(){

			$this->start_controls_section(
				'section_carousel_settings',
				[
					'label' => __( 'Carousel Settings', 'rey-core' ),
				]
			);

			$items_to_show = range( 1, 10 );
			$items_to_show = array_combine( $items_to_show, $items_to_show );

			$this->add_responsive_control(
				'items_to_show',
				[
					'label' => __( 'Items to Show', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::SELECT,
					'options' => [
						'' => __( 'Default', 'rey-core' ),
					] + $items_to_show,
					'selectors' => [
						$this->selectors['carousel'] => '--per-row: {{VALUE}}',
					],
					'default' => 4,
					'devices' => [ 'desktop', 'tablet', 'mobile' ],
					'desktop_default' => 4,
					'tablet_default' => 3,
					'mobile_default' => 2,
				]
			);

			$this->add_responsive_control(
				'gap',
				[
					'label' => __( 'Gap', 'rey-core' ) . ' (px)',
					'type' => \Elementor\Controls_Manager::NUMBER,
					'default' => '',
					'min' => 0,
					'max' => 200,
					'step' => 1,
					'selectors' => [
						$this->selectors['carousel'] => '--gap: {{VALUE}}px;',
					],
				]
			);

			$this->add_control(
				'infinite',
				[
					'label' => __( 'Infinite Loop', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::SWITCHER,
					'default' => 'yes',
				]
			);

			$this->add_control(
				'autoplay',
				[
					'label' => __( 'Autoplay', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::SWITCHER,
					'return_value' => 'yes',
					'default' => '',
				]
			);

			$this->add_control(
				'autoplay_duration',
				[
					'label' => __( 'Autoplay Duration (ms)', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::NUMBER,
					'default' => 9000,
					'min' => 3500,
					'max' => 20000,
					'step' => 50,
					'condition' => [
						'autoplay' => 'yes',
					],
					'selectors' => [
						$this->selectors['carousel'] => '--autoplay-duration: {{SIZE}}ms;',
					],
				]
			);

			$this->add_control(
				'autoplay_pause_hover',
				[
					'label' => esc_html__( 'Pause on hover', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::SWITCHER,
					'default' => 'yes',
					'condition' => [
						'autoplay!' => '',
					],
				]
			);

			$this->add_control(
				'arrows',
				[
					'label' => __( 'Arrows Navigation', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::SWITCHER,
					'default' => 'yes',
				]
			);

			$this->add_control(
				'dots',
				[
					'label' => __( 'Dots Navigation', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::SWITCHER,
					'default' => 'yes',
				]
			);

			$this->add_responsive_control(
				'c_padding',
				[
					'label' => __( 'Padding', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::DIMENSIONS,
					'description' => 'Adding left or right padding will cutoff the next slides.',
					'selectors' => [
						$this->selectors['wrapper'] . ' .splide__track' => 'padding-top: {{TOP}}px; padding-left: {{LEFT}}px; padding-right: {{RIGHT}}px; padding-bottom: {{BOTTOM}}px;',
					],
					'render_type' => 'template',
				]
			);

			$this->add_control(
				'carousel_id',
				[
					'label' => __( 'Carousel Unique ID', 'rey-core' ),
					'label_block' => true,
					'type' => \Elementor\Controls_Manager::TEXT,
					'default' => uniqid('carousel-'),
					'placeholder' => __( 'eg: some-unique-id', 'rey-core' ),
					'description' => __( 'Copy the ID above and paste it into the "Toggle Boxes" Widget or "Slider Navigation" widget where specified. No hashtag needed. Read more on <a href="https://support.reytheme.com/kb/products-grid-element/#adding-custom-navigation" target="_blank">how to connect them</a>.', 'rey-core' ),
					'separator' => 'before'
				]
			);

			$this->end_controls_section();

		}

		public function controls__arrow_styles(){

			$this->start_controls_section(
				'section_arrow_style',
				[
					'label' => __( 'Arrows Nav. Styles', 'rey-core' ),
					'tab' => \Elementor\Controls_Manager::TAB_STYLE,
					'condition' => [
						'arrows!' => '',
					],
				]
			);

			$this->add_control(
				'arrows_position',
				[
				   'label' => esc_html__( 'Arrows Vertical Position', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::SLIDER,
					'range' => [
						'%' => [
							'min' => 0,
							'max' => 100,
							'step' => 1,
						],
					],
					'selectors' => [
						$this->selectors['carousel'] => '--arrows-v-pos: {{SIZE}}%;',
					],
				]
			);

			$this->add_control(
				'arrows_on_hover',
				[
					'label' => __( 'Show Arrows On Hover', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::SWITCHER,
					'default' => 'yes',
				]
			);

			$this->add_responsive_control(
				'arrows_spacing',
				[
					'label' => esc_html__( 'Arrows Spacing', 'rey-core' ) . ' (px)',
					'type' => \Elementor\Controls_Manager::NUMBER,
					'default' => '',
					'min' => -100,
					'max' => 1000,
					'step' => 1,
					'selectors' => [
						$this->selectors['carousel'] => '--arrow-side-spacing:{{VALUE}}px;',
					],
				]
			);

			$this->add_control(
				'arrows_type',
				[
					'label' => esc_html__( 'Arrows Type', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::SELECT,
					'default' => '',
					'options' => [
						''  => esc_html__( 'Default', 'rey-core' ),
						'chevron'  => esc_html__( 'Chevron', 'rey-core' ),
						'custom'  => esc_html__( 'Custom Icon', 'rey-core' ),
					],
				]
			);

			$this->add_control(
				'arrows_custom_icon',
				[
					'label' => __( 'Custom Arrow Icon (Right)', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::ICONS,
					'condition' => [
						'arrows_type' => 'custom',
					],
				]
			);

			$this->add_control(
				'arrows_size',
				[
					'label' => esc_html__( 'Arrows size', 'rey-core' ) . ' (px)',
					'type' => \Elementor\Controls_Manager::NUMBER,
					'default' => '',
					'min' => 5,
					'max' => 200,
					'step' => 1,
					'selectors' => [
						$this->selectors['carousel'] => '--arrow-size: {{VALUE}}px',
					],
				]
			);

			$this->add_control(
				'arrows_height',
				[
					'label' => esc_html__( 'Arrows height', 'rey-core' ) . ' (em)',
					'type' => \Elementor\Controls_Manager::NUMBER,
					'default' => '',
					'min' => 0.1,
					'max' => 1,
					'step' => 0.1,
					'selectors' => [
						$this->selectors['arrow'] => '--arrow-height: {{VALUE}}em',
					],
				]
			);

			$this->add_responsive_control(
				'arrows_padding',
				[
					'label' => __( 'Arrows Padding', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::DIMENSIONS,
					'size_units' => [ 'px', 'em', '%' ],
					'selectors' => [
						$this->selectors['arrow'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
					],
				]
			);

			$this->start_controls_tabs( 'arrows_tabs_styles');

				$this->start_controls_tab(
					'arrows_tab_default',
					[
						'label' => __( 'Default', 'rey-core' ),
					]
				);

					$this->add_control(
						'arrows_color_diff',
						[
							'label' => esc_html__( 'Enable difference color?', 'rey-core' ),
							'type' => \Elementor\Controls_Manager::SWITCHER,
							'default' => 'difference',
							'return_value' => 'difference',
							'selectors' => [
								$this->selectors['arrow'] => 'mix-blend-mode: {{VALUE}}',
							],
						]
					);

					$this->add_control(
						'arrows_color',
						[
							'label' => esc_html__( 'Arrows Color', 'rey-core' ),
							'type' => \Elementor\Controls_Manager::COLOR,
							'selectors' => [
								$this->selectors['arrow'] => 'color: {{VALUE}}',
							],
						]
					);

					$this->add_control(
						'arrows_bg_color',
						[
							'label' => esc_html__( 'Arrows Background Color', 'rey-core' ),
							'type' => \Elementor\Controls_Manager::COLOR,
							'selectors' => [
								$this->selectors['arrow'] => 'background-color: {{VALUE}}',
							],
						]
					);

					$this->add_group_control(
						\Elementor\Group_Control_Border::get_type(),
						[
							'name' => 'arrows_border',
							'selector' => $this->selectors['arrow'],
							'responsive' => true,
						]
					);

				$this->end_controls_tab();

				$this->start_controls_tab(
					'arrows_tab_hover',
					[
						'label' => __( 'Hover', 'rey-core' ),
					]
				);

					$this->add_control(
						'arrows_color_hover',
						[
							'label' => esc_html__( 'Arrows Color', 'rey-core' ),
							'type' => \Elementor\Controls_Manager::COLOR,
							'selectors' => [
								$this->selectors['arrow'] . ':hover' => 'color: {{VALUE}}',
							],
						]
					);

					$this->add_control(
						'arrows_bg_color_hover',
						[
							'label' => esc_html__( 'Arrows Background Color', 'rey-core' ),
							'type' => \Elementor\Controls_Manager::COLOR,
							'selectors' => [
								$this->selectors['arrow'] . ':hover' => 'background-color: {{VALUE}}',
							],
						]
					);

					$this->add_control(
						'arrows_border_color_hover',
						[
							'label' => esc_html__( 'Arrows Border Color', 'rey-core' ),
							'type' => \Elementor\Controls_Manager::COLOR,
							'selectors' => [
								$this->selectors['arrow'] . ':hover' => 'border-color: {{VALUE}}',
							],
						]
					);

				$this->end_controls_tab();

			$this->end_controls_tabs();

			$this->add_control(
				'arrows_border_radius',
				[
					'label' => __( 'Border Radius', 'rey-core' ),
					'type' => \Elementor\Controls_Manager::DIMENSIONS,
					'size_units' => [ 'px', '%' ],
					'selectors' => [
						$this->selectors['arrow'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
					],
				]
			);

			$this->end_controls_section();
		}

		public function controls__dots_styles(){

			$this->start_controls_section(
				'section_dots_style',
				[
					'label' => __( 'Dots Nav. Styles', 'rey-core' ),
					'tab' => \Elementor\Controls_Manager::TAB_STYLE,
					'condition' => [
						'dots!' => '',
					],
				]
			);

				$this->add_control(
					'dots_color',
					[
						'label' => esc_html__( 'Dots Color', 'rey-core' ),
						'type' => \Elementor\Controls_Manager::COLOR,
						'selectors' => [
							'{{WRAPPER}} .__nav' => 'color: {{VALUE}}',
						],
					]
				);

				$this->add_control(
					'dots_color_diff',
					[
						'label' => esc_html__( 'Enable difference color?', 'rey-core' ),
						'type' => \Elementor\Controls_Manager::SWITCHER,
						'default' => 'difference',
						'return_value' => 'difference',
						'selectors' => [
							'{{WRAPPER}} .__nav' => 'mix-blend-mode: {{VALUE}}',
						],
					]
				);

				$this->add_control(
					'dots_style',
					[
						'label' => esc_html__( 'Dots Style', 'rey-core' ),
						'type' => \Elementor\Controls_Manager::SELECT,
						'default' => 'circles',
						'options' => [
							'circles'  => esc_html__( 'Circles', 'rey-core' ),
							// 'circles-ext'  => esc_html__( 'Dots & Circle', 'rey-core' ),
							'bars'  => esc_html__( 'Bars', 'rey-core' ),
						],
						'condition' => [
							'dots!' => '',
						],
					]
				);

				$this->add_control(
					'dots_position',
					[
						'label' => esc_html__( 'Dots Position', 'rey-core' ),
						'type' => \Elementor\Controls_Manager::SELECT,
						'default' => 'bottom-center',
						'options' => [
							'top-left'  => esc_html__( 'Top Left', 'rey-core' ),
							'top-center'  => esc_html__( 'Top Center', 'rey-core' ),
							'top-right'  => esc_html__( 'Top Right', 'rey-core' ),
							'bottom-left'  => esc_html__( 'Bottom Left', 'rey-core' ),
							'bottom-center'  => esc_html__( 'Bottom Center', 'rey-core' ),
							'bottom-right'  => esc_html__( 'Bottom Right', 'rey-core' ),
						],
						'condition' => [
							'dots!' => '',
						],
					]
				);

				$this->add_responsive_control(
					'dots_spacing',
					[
						'label' => esc_html__( 'Spacing', 'rey-core' ) . ' (px)',
						'type' => \Elementor\Controls_Manager::NUMBER,
						'default' => '',
						'min' => -100,
						'max' => 1000,
						'step' => 1,
						'selectors' => [
							$this->selectors['carousel'] => '--dots-side-spacing:{{VALUE}}px;',
						],
					]
				);

			$this->end_controls_section();
		}

		public function render_start(){

			$classes = [
				'rey-carouselEl',
				'--' . $this->_settings['source'],
			];

			if( $this->_settings['autoplay'] !== '' && $this->_settings['autoplay_pause_hover'] !== '' ){
				$classes[] = '--pause-hover';
			}

			$this->add_render_attribute( 'wrapper', 'class', $classes );

			$carousel_config = [
				'type'             => 'slide',
				'items_to_show'    => $this->_settings['items_to_show'],
				'gap'              => $this->_settings['gap'],
				'infinite'         => $this->_settings['infinite'] !== '',
				'autoplay'         => $this->_settings['autoplay'] !== '',
				'pauseOnHover'     => $this->_settings['autoplay'] !== '' && $this->_settings['autoplay_pause_hover'] !== '',
				'interval'         => absint($this->_settings['autoplay_duration']),
				'customArrows'     => '.__arrows-' . $this->get_id(),
				'customPagination' => '.__pagination-' . $this->get_id(),
				'speed'            => 700,
				'c_padding'        => $this->_settings['c_padding'],
				'c_padding_tablet' => isset($this->_settings['c_padding_tablet']) ? $this->_settings['c_padding_tablet'] : '',
				'c_padding_mobile' => isset($this->_settings['c_padding_mobile']) ? $this->_settings['c_padding_mobile'] : '',
			];

			foreach ( ['items_to_show', 'gap'] as $val ) {
				foreach ( ['_mobile', '_tablet'] as $dev ) {
					if( isset($this->_settings[ $val . $dev ]) ){
						$carousel_config[ $val . $dev ] = $this->_settings[ $val . $dev ];
					}
				}
			}

			if( count($this->_items) > 1 ) {
				$this->add_render_attribute( 'wrapper', 'data-carousel-settings', wp_json_encode($carousel_config) );
			}
			?><div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>><?php
		}

		public function render_end(){
			?></div><?php
		}

		public function render_carousel(){

			if( empty($this->_items) ){
				return;
			}

			$classes = [];

			if( $carousel_id = esc_attr( $this->_settings['carousel_id'] ) ){
				$classes[] =  $carousel_id;
			} ?>

			<div class="splide <?php echo esc_attr(implode(' ', $classes)) ?>">
				<div class="splide__track">
					<div class="splide__list __slides">

						<?php
						for ($i=0; $i < count($this->_items); $i++) {
							$this->item_key = $i;

							$this->parse_item();
							$this->render_item__start();
							$this->render_item();
							$this->render_item__end();

						} ?>

					</div>
				</div>
			</div>
			<?php
		}

		public function render_item__start(){

			$classes = [
				'splide_slide' => 'splide__slide',
				'item' => '__slide',
			];

			if( isset($this->_items[$this->item_key]['_id']) && $_id = $this->_items[$this->item_key]['_id'] ){
				$classes['_id'] = 'elementor-repeater-item-' . $_id;
			}

			?><div class="<?php echo esc_attr(implode(' ', $classes)) ?>"><?php
		}

		public function render_item__end(){
			?></div><?php
		}

		public function render_arrows() {

			if( empty($this->_items) ){
				return;
			}

			if( $this->_settings['arrows'] === 'yes' ):

				$classes[] = '__arrows-' . $this->get_id();
				$classes[] = $this->_settings['arrows_on_hover'] === 'yes' ? '--hide-on-idle' : '';
				?>
				<div class="__arrows <?php echo implode(' ', $classes) ?>">
					<?php
					$custom_svg_icon = '';

					if( 'custom' === $this->_settings['arrows_type'] &&
						($custom_icon = $this->_settings['arrows_custom_icon']) &&
						isset($custom_icon['value']) && !empty($custom_icon['value'])
					){
						ob_start();
						\Elementor\Icons_Manager::render_icon( $custom_icon, [ 'aria-hidden' => 'true', 'class' => '' ] );
						$custom_svg_icon = ob_get_clean();
					}

					reycore__svg_arrows([
						'type' => $this->_settings['arrows_type'],
						'custom_icon' => $custom_svg_icon,
						'attributes' => [
							'left' => 'data-dir="-1"',
							'right' => 'data-dir="+1"',
						]
					]); ?>
				</div>
			<?php endif;
		}

		public function render_nav(){

			if( empty($this->_items) ){
				return;
			}

			if( $this->_settings['dots'] === 'yes' ){

				$classes[] = '__pagination-' . $this->get_id();
				$classes[] = '--dots-pos--' . $this->_settings['dots_position'];
				$classes[] = '--dots-style--' . $this->_settings['dots_style'];

				if( $this->_settings['autoplay'] !== '' ){
					$classes[] = '--autoplay';
				}

				?>
				<div class="__nav <?php  echo implode(' ', $classes); ?>">
					<?php
					foreach($this->_items as $key => $item):
						printf('<button data-go="%d" class="%s"></button>', esc_attr($key), $key === 0 ? 'is-active' : '');
					endforeach; ?>
				</div>
			<?php }
		}

		public function render() {

			reyCoreAssets()->add_styles(['reycore-widget-carousel-styles', 'rey-splide']);
			reyCoreAssets()->add_scripts( $this->rey_get_script_depends() );

			$this->_settings = $this->get_settings_for_display();
			$this->_items = $this->get_items_data();

			$this->render_start();
			$this->render_carousel();
			$this->render_nav();
			$this->render_arrows();
			$this->render_end();
		}

	}
endif;
