<?php

namespace ReyCore\ScheduledSales;

use ReyCore\ScheduledSales\Base as Base;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Catalog
{
	public function __construct()
	{
		add_action('init', [$this, 'init']);
	}

	public function init(){

		$this->instance = instance();

		if( ! ($this->type = $this->instance->data['loop']['type']) ){
			return;
		}

		add_filter( 'reycore/loop_components', [$this, 'enable_component']);
		add_filter( 'reycore/loop/component_hooks', [$this, 'hook_component']);
	}

	function enable_component( $components ){
		$components['scheduled_sale'] = true;
		return $components;
	}

	function hook_component( $components ){

		$position = $this->instance->data['loop']['position'];

		$hooks = [
			'top_left' => [
				'hook'     => 'reycore/loop_inside_thumbnail/top-left',
				'priority' => 10,
			],
			'top_right' => [
				'hook'     => 'reycore/loop_inside_thumbnail/top-right',
				'priority' => 10,
			],
			'bottom_left' => [
				'hook'     => 'reycore/loop_inside_thumbnail/bottom-left',
				'priority' => 10,
			],
			'bottom_right' => [
				'hook'     => 'reycore/loop_inside_thumbnail/bottom-right',
				'priority' => 10,
			],
			'before_title' => [
				'hook'     => 'woocommerce_before_shop_loop_item_title',
				'priority' => 20,
			],
			'after_title' => [
				'hook'     => 'woocommerce_after_shop_loop_item_title',
				'priority' => 10,
			],
			'after_content' => [
				'hook'     => 'woocommerce_after_shop_loop_item',
				'priority' => 905,
			],
		];

		if( ! isset($hooks[ $position ]) ){
			return $components;
		}

		// countdown is not fitting inside the thumb badge slots
		if( $this->type === 'countdown' ){
			$hooks['top_left']     = $hooks['before_title'];
			$hooks['top_right']    = $hooks['before_title'];
			$hooks['bottom_left']  = $hooks['before_title'];
			$hooks['bottom_right'] = $hooks['before_title'];
		}

		$components['scheduled_sale'] = [
			'type'          => 'action',
			'tag'           => $hooks[ $position ]['hook'],
			'callback'      => [ $this->instance, 'render' ],
			'priority'      => $hooks[ $position ]['priority'],
		];

		return $components;
	}

}

new Catalog();
