<?php

namespace ReyCore\VariationSwatches;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Base
{
	private static $_instance = null;

	public static $settings = [];

	const ASSET_HANDLE = 'reycore-variation-swatches';

	const OPT = 'rey_swatches_data';

	const FIELDS_PREFIX = 'rey__';

	const TRANSIENT__ATTRIBUTE_SWATCHES_SETTINGS = 'rey_attribute_swatches_settings_';

	const TYPES_PREFIX = 'rey_';

	public $swatches = [];

	private function __construct()
	{
		add_action('init', [$this, 'init']);
		$this->includes();
	}

	function includes(){

		require_once __DIR__ . '/admin.php';
		require_once __DIR__ . '/migration.php';
		require_once __DIR__ . '/frontend.php';
		require_once __DIR__ . '/compat.php';
		require_once __DIR__ . '/impex.php';
		require_once __DIR__ . '/types.php';
		require_once __DIR__ . '/types-button.php';
		require_once __DIR__ . '/types-image.php';
		require_once __DIR__ . '/types-color.php';
		require_once __DIR__ . '/types-radio.php';
		require_once __DIR__ . '/types-large-button.php';

	}

	public function is_enabled(){

		$enabled = class_exists('WooCommerce');

		if(
			class_exists('Woo_Variation_Swatches')
			|| class_exists('Woo_Variation_Swatches_Pro')
			|| class_exists('Iconic_Woo_Attribute_Swatches')
			|| class_exists('VIWPVS_WOOCOMMERCE_PRODUCT_VARIATIONS_SWATCHES')
			|| class_exists('WC_SwatchesPlugin') // WooCommerce Variation Swatches and Photos
		){
			$enabled = false;
		}

		return apply_filters('reycore/variation_swatches/enabled', $enabled );
	}

	function init(){

		self::$settings = apply_filters('reycore/variation_swatches/settings', [
			'tooltip_theme'                     => 'light',
			'tooltip_image_size'                => 'thumbnail',
			'variations_use_main_image'         => false,
			'optimized_assets_load'             => true,
			'show_all_labels'                   => false,
			'always_show_migration'             => false,
		] );

		self::$settings['enabled'] = $this->is_enabled();

		do_action('reycore/variation_swatches/init', $this);

	}

	function register_swatch_type( $swatch_class ){
		if( $swatch_id = $swatch_class->get_id() ){
			$this->swatches[ $swatch_id ] = $swatch_class;
		}
	}

	public function swatch_exists( $type ){
		$types = $this->get_swatches_list();
		return isset( $types[ $type ] );
	}

	public function get_swatches_list(){

		$swatches = [];

		foreach ($this->swatches as $id => $swatch) {
			$swatches[ $id ] = $swatch->get_name();
		}

		return $swatches;
	}

	public function get_swatches( $swatch_id = '' ){

		if( $swatch_id && isset( $this->swatches[ $swatch_id ] ) ){
			return $this->swatches[ $swatch_id ];
		}

		return $this->swatches;
	}

	public function get_default_swatch(){

		$default_type = '';

		foreach ($this->swatches as $id => $swatch) {
			if( $swatch->is_default ){
				$default_type = $id;
			}
		}

		return $default_type;
	}

	static function get_attributes_swatch_settings( $tax = '' ){

		$opt = get_option(self::OPT, []);

		if( $tax ){
			if( isset($opt[ $tax ]) ){
				return $opt[ $tax ];
			}
			return [];
		}

		return $opt;
	}

	static function set_attributes_swatch_settings($data){
		return update_option(self::OPT, $data);
	}

	static function is_single_product(){

		if( reycore_wc__is_product() ){
			return ! in_array( wc_get_loop_prop('name'), ['upsells', 'up-sells', 'crosssells', 'cross-sells', 'related'] );
		}

		return false;
	}

	public static function get_term($term, $taxonomy = null){

		if ( empty( $term ) ) {
			return;
		}

		if ( $term instanceof \WP_Term ) {
			$_term = $term;
		} elseif ( is_object( $term ) && isset($term->term_id) ) {
			$_term = \WP_Term::get_instance( $term->term_id );
		} else {
			if( ! $_term = \WP_Term::get_instance( $term, $taxonomy ) ){
				$_term = get_terms( ['include' => (array) $term] );
			}
		}

		if( is_wp_error($_term) ){
			return;
		}

		return $_term;
	}


	/**
	 * Retrieve the reference to the instance of this class
	 * @return Base
	 */
	public static function getInstance()
	{
		if ( is_null( self::$_instance ) || ! ( self::$_instance instanceof self ) ) {
			self::$_instance = new self;
		}
		return self::$_instance;
	}

}

function instance(){
	return Base::getInstance();
}

instance();
