<?php
// exit if accessed directly
if( ! defined( 'ABSPATH' ) ) exit;

if( !class_exists('ReyCore_WooCommerce_Variations') ):

class ReyCore_WooCommerce_Variations
{
	private static $_instance = null;

	const TRANSIENT_PRODUCT_VARIATIONS = 'rey_product_variations_';

	public $store_transients = true;

	public $selected_attribute_taxonomy;
	public $selected_attribute_taxonomy_data = [];

	private function __construct()
	{

		if(defined('REY_DEV_MODE') && REY_DEV_MODE){
			$this->store_transients = false;
		}

		add_filter( 'reycore/loop/component_hooks', [$this, 'loop_component']);
		add_filter( 'reycore/kirki_fields/field=woocommerce_loop_variation', [ $this, 'customizer__add_attributes_choices' ], 11 );
		add_filter( 'reycore/kirki_fields/field=pdp_swatches__update_gallery', [ $this, 'customizer__add_attributes_choices' ], 11 );
		// Clean transients
		add_action( 'customize_save_woocommerce_loop_variation', [$this, 'customizer__clear_transients']);
		add_action( 'customize_save_woocommerce_loop_variation_limit', [$this, 'customizer__clear_transients']);
		add_action( 'woocommerce_delete_product_transients', [$this, 'clear_product_transient']);
		add_action( 'save_post', [$this, 'clear_product_transient'] );
		add_action( 'woocommerce_attribute_updated', [ $this, 'clean_db_transients' ] );
		add_action( 'woocommerce_attribute_deleted', [$this, 'clean_db_transients'] );
	}

	function loop_component( $components ){

		$position = get_theme_mod('woocommerce_loop_variation_position', 'after');

		$hooks = [
			'first' => [
				'hook'     => 'woocommerce_before_shop_loop_item_title',
				'priority' => 20,
			],
			'before' => [
				'hook'     => 'woocommerce_after_shop_loop_item',
				'priority' => 5,
			],
			'after' => [
				'hook'     => 'woocommerce_after_shop_loop_item',
				'priority' => 905,
			],
		];

		if( ! isset($hooks[ $position ]) ){
			return $components;
		}

		$components['variations'] = [
			'type'          => 'action',
			'tag'           => $hooks[ $position ]['hook'],
			'callback'      => [ $this, 'render_attribute' ],
			'priority'      => $hooks[ $position ]['priority'],
		];

		return $components;
	}

	function render_attribute() {

		global $product;

		if ( ! ($product && 'variable' === $product->get_type()) ) {
			return;
		}

		if( ! $this->selected_attribute_taxonomy ){
			if( ! ($this->selected_attribute_taxonomy = $this->get_selected_attribute_taxonomy()) ){
				return;
			}
		}

		if( 'disabled' === $this->selected_attribute_taxonomy ){
			return;
		}


		// Render All attributes
		if( 'all_attributes' === $this->selected_attribute_taxonomy ){
			do_action('reycore/woocommerce/variations/catalog/render_all', $product, $this);
			return;
		}

		if( ! $this->selected_attribute_taxonomy_data ){
			if( ! ($this->selected_attribute_taxonomy_data = $this->get_attribute_taxonomy_data()) ){
				return;
			}
		}

		if( ($display_text = get_theme_mod('woocommerce_loop_variation_force_regular', '')) &&
			($regular_attribute = $this->get_regular_attribute_data( $product, $display_text )) ){
			echo $regular_attribute;
			return;
		}

		// Render Single chosen attribute
		$this->render_attribute__single( $product );
	}

	function render_attribute__single( $product ){

		if( empty( $this->selected_attribute_taxonomy_data ) ){
			return;
		}

		do_action('reycore/woocommerce/variations/catalog/render_single', $product, $this);
	}

	/**
	 * Get the selected visible attribute data
	 *
	 * @since 1.0.0
	 */
	function get_attribute_taxonomy_data()
	{
		$data['name'] = $this->selected_attribute_taxonomy;
		$data['taxonomy_name'] = wc_attribute_taxonomy_name( $data['name'] );
		$data['type'] = 'select';
		return apply_filters('reycore/woocommerce/variations/variation_attribute_data', $data);
	}


	/**
	 * Gets a regular attribute data, regardless if
	 * product is variable
	 *
	 * @since 1.3.9
	 */
	function get_regular_attribute_data( $product, $display_type ){

		$attr_data = $this->selected_attribute_taxonomy_data;
		$all_attributes = $product->get_attributes();

		if( ! (isset( $all_attributes[$attr_data['taxonomy_name']] ) && $attribute = $all_attributes[$attr_data['taxonomy_name']]) ){
			return;
		}

		if( ! is_object($attribute) ){
			return;
		}

		if( ! $attribute->get_visible() ){
			return;
		}

		$custom_text = get_theme_mod('woocommerce_loop_variation_text_display', '');

		// count
		if( 'count' === $display_type ){

			$text = $custom_text ? $custom_text : esc_html__('items', 'rey-core');

			if( ($options = $attribute->get_options()) && !empty($options) && count($options) > 1 ){
				return sprintf(
					'<div class="rey-productAttribute"><span class="__count">%s</span> <span class="__text">%s</span></div>',
					count($options),
					$text
				);
			}
		}
		// text list
		else {

			$tax = $attribute->get_name();
			$attribute_values   = wc_get_product_terms( $product->get_id(), $tax, [ 'fields' => 'all' ] );

			$values = [];

			foreach ( $attribute_values as $attribute_value ) {
				$values[] = esc_html( ucfirst( $attribute_value->name ) );
			}

			$text = $custom_text ? $custom_text : strtoupper(wc_attribute_label( $tax ));

			if( count($values) > 1 ){
				return sprintf(
					'<div class="rey-productAttribute">' . apply_filters('reycore/woocommerce/loop_attribute_display', '%s: %s', $attribute, $values) . '</div>',
					$text,
					implode( ', ', $values )
				);
			}
		}

	}

	function get_selected_attribute_taxonomy(){
		return get_theme_mod('woocommerce_loop_variation', 'disabled');
	}

	function customizer__add_attributes_choices( $args ){
		$args['choices'] = array_merge($args['choices'], reycore_wc__get_attributes_list());
		return $args;
	}

	/**
	 * *****************************************************
	 */

	function clean_db_transients(){

		global $wpdb;

		$like_main = '%transient_' . $wpdb->esc_like( self::TRANSIENT_PRODUCT_VARIATIONS ) . '%';
		$like_timeout = '%transient_timeout_' . $wpdb->esc_like( self::TRANSIENT_PRODUCT_VARIATIONS ) . '%';
		$query = $wpdb->prepare( "DELETE FROM {$wpdb->options} WHERE option_name LIKE %s OR option_name LIKE %s ", $like_main, $like_timeout );

		return $wpdb->query($query);
	}

	/**
	 * Clear some transients on Customizer Publish
	 *
	 * @since 1.0.0
	 */
	function customizer__clear_transients( $setting )
	{

		if( ! current_user_can('administrator') ){
			return;
		}

		if( ! method_exists($setting, 'value') ){
			return;
		}

		$setting_data = $setting->id_data();
		// old
		$value = $setting->value();
		// new
		$post_value = $setting->post_value();

		$clear = false;

		if( $value === $post_value ){
			return;
		}

		if( ! isset($setting_data['base']) ){
			return;
		}

		if( 'woocommerce_loop_variation' === $setting_data['base'] ){
			if( $post_value !== 'disabled' ){
				$clear = true;
			}
		}

		elseif( 'woocommerce_loop_variation_limit' === $setting_data['base'] ){
			$clear = true;
		}

		if( $clear ){
			$this->clean_db_transients();
		}
	}

	static function transient_name( $product_id, $v2 = true ){
		return self::TRANSIENT_PRODUCT_VARIATIONS . ($v2 ? '_v2_' : '') . $product_id;
	}

	function clear_product_transient( $post_id = 0 ){

		if ( $post_id === 0 ) {
			$this->clean_db_transients();
		}

		if ( $post_id > 0 ) {

			if( function_exists('rey__maybe_disable_obj_cache') ){
				rey__maybe_disable_obj_cache();
			}

			delete_transient( self::transient_name($post_id) );
			delete_transient( self::transient_name($post_id, false) );

		}
	}

	/**
	 * Legacy
	 */

	function do_variation(){
		$this->render_attribute();
	}

	/**
	 * Retrieve the reference to the instance of this class
	 * @return ReyCore_WooCommerce_Variations
	 */
	public static function getInstance()
	{
		if ( is_null( self::$_instance ) || ! ( self::$_instance instanceof self ) ) {
			self::$_instance = new self;
		}
		return self::$_instance;
	}

}
endif;

ReyCore_WooCommerce_Variations::getInstance();
