<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
if( !class_exists('ReyTheme_Setup') ):

	class ReyTheme_Setup
	{

		public function __construct()
		{
			add_action( 'after_switch_theme', [$this, 'check_theme']);
			add_action( 'after_setup_theme', [$this, 'after_setup_theme']);
			add_action( 'after_setup_theme', [$this, 'set_content_width'], 0);
			add_action( 'widgets_init', [$this, 'widgets_init']);
			add_action( 'init', [$this, 'init'], 0);
		}

		/**
		 * Disable theme if PHP 5.4 not supported & WP Version is 4.7+
		 */
		function check_theme(){

			/**
			 * PHP Version Check.
			 */
			if ( version_compare( PHP_VERSION, REY_THEME_REQUIRED_PHP_VERSION, '<' ) ) :
				// Theme not activated info message
				add_action( 'admin_notices', 'rey__php_version_admin_notice' );
				function rey__php_version_admin_notice() {
					?>
					<div class="notice notice-error">
						<?php printf( esc_html__( 'This theme requires a minimum PHP version of %s. Your version is s%. Your previous theme has been restored.', 'rey' ), REY_THEME_REQUIRED_PHP_VERSION, PHP_VERSION ); ?>
					</div>
					<?php
				}
				// Switch back to previous theme
				switch_theme( get_option( 'theme_switched' ) );
				return false;
			endif;

		}

		/**
		 * Sets up theme defaults and registers support for various WordPress features.
		 *
		 * Note that this function is hooked into the after_setup_theme hook, which
		 * runs before the init hook. The init hook is too late for some features, such
		 * as indicating support for post thumbnails.
		 */
		function after_setup_theme() {

			/*
			 * Make theme available for translation.
			 * Translations can be filed in the /languages/ directory.
			 * If you're building a theme based on components, use a find and replace
			 * to change 'rey' to the name of your theme in all the template files.
			 */
			load_theme_textdomain( 'rey', REY_THEME_DIR . '/languages' );

			// Add default posts and comments RSS feed links to head.
			add_theme_support( 'automatic-feed-links' );

			/*
			 * Let WordPress manage the document title.
			 * By adding theme support, we declare that this theme does not use a
			 * hard-coded <title> tag in the document head, and expect WordPress to
			 * provide it for us.
			 */
			add_theme_support( 'title-tag' );

			/*
			 * Enable support for Post Thumbnails on posts and pages.
			 *
			 * @link https://developer.wordpress.org/themes/functionality/featured-images-post-thumbnails/
			 */
			add_theme_support( 'post-thumbnails' );
			set_post_thumbnail_size( 1440, 9999 , true );
			add_image_size( 'rey-standard-large', 1024, 9999 );
			add_image_size( 'rey-ratio-16-9', 1440, 810, true ); // height = 1440 x 0.5625


			// This theme uses wp_nav_menu() in one location.
			register_nav_menus( array(
				'main-menu' => esc_html__( 'Main Menu', 'rey' ),
				'footer-menu' => esc_html__( 'Footer Menu', 'rey' ),
			));

			/**
			 * Add support for core custom logo.
			 */
			add_theme_support( 'custom-logo', array(
				'height'      => 200,
				'width'       => 200,
				'flex-width'  => true,
				'flex-height' => true,
			) );

			/*
			 * Switch default core markup for search form, comment form, and comments
			 * to output valid HTML5.
			 */
			add_theme_support( 'html5', array(
				'search-form',
				'comment-form',
				'comment-list',
				'gallery',
				'caption',
			) );

			// add support for post formats
			add_theme_support('post-formats', ['gallery', 'image', 'video', 'audio', 'link', 'quote', 'status']);

			// Gutenberg Editor
			add_theme_support( 'align-wide' );

			/*
			 * This theme styles the visual editor to resemble the theme style,
			 * specifically font, colors, and column width.
			  */
			add_theme_support( 'editor-styles' );
			add_editor_style( 'assets/css/editor.css' );

			do_action('rey/after_setup_theme');

		}

		/**
		 * Set the content width in pixels, based on the theme's design and stylesheet.
		 *
		 * Priority 0 to make it available to lower priority callbacks.
		 *
		 * @global int $content_width
		 */
		function set_content_width() {
			$GLOBALS['content_width'] = $content_width = apply_filters( 'rey/content/width', 1440 );
		}

		/**
		 * Register widget area.
		 */
		function widgets_init() {

			register_sidebar( [
				'name'          => esc_html__( 'Sidebar', 'rey' ),
				'id'            => 'main-sidebar',
				'description'   => esc_html__('This sidebar will be visible on the pages with default template option.' , 'rey'),
				'before_widget' => '<section id="%1$s" class="widget %2$s">',
				'after_widget'  => '</section>',
				'before_title'  => '<h3 class="widget-title">',
				'after_title'   => '</h3>',
			] );

			do_action('rey/widgets_init');

		}

		function init(){
			do_action('rey/init');
		}

	}

	new ReyTheme_Setup;

endif;
